@@ -3,12 +3,11 @@
 use utf8;
 
 # Export.pm -- Export EekBoek administratie
-# RCS Info        : $Id: Export.pm,v 1.31 2009/10/15 10:05:03 jv Exp $
 # Author          : Johan Vromans
 # Created On      : Mon Jan 16 20:47:38 2006
 # Last Modified By: Johan Vromans
-# Last Modified On: Thu Oct 15 11:35:50 2009
-# Update Count    : 233
+# Last Modified On: Mon Jan 16 15:11:32 2012
+# Update Count    : 243
 # Status          : Unknown, Use with caution!
 
 package main;
@@ -21,8 +20,6 @@ package EB::Export;
 use strict;
 use warnings;
 
-our $VERSION = sprintf "%d.%03d", q$Revision: 1.31 $ =~ /(\d+)/g;
-
 use EB;
 use EB::Format;
 use Encode;
@@ -141,9 +138,9 @@ sub _relaties {
 
     my $cur_dbk = "";
     my $cur_btw = -1;
-    my $out = __x("# {what} voor administratie {adm}",
-		  what => _T("Relaties"), adm => $dbh->adm("name")) . "\n" .
-	      __x("# Aangemaakt door {id} op {date}",
+    my $out = "# " . __x("{what} voor administratie {adm}",
+			 what => _T("Relaties"), adm => $dbh->adm("name")) . "\n" .
+	      "# " . __x("Aangemaakt door {id} op {date}",
 		  id => $EB::ident, date => datefmt_full(iso8601date())) . "\n" .
 	      "# Content-Type: text/plain; charset = UTF-8";
     while ( my $rr = $sth->fetchrow_arrayref ) {
@@ -154,8 +151,8 @@ sub _relaties {
 	    $cur_dbk = $dbk;
 	    $dbk =~ s/[^[:alnum]]/_/g;
 	    $out .= "\n\n" if $out;
-	    $out .= "relatie --dagboek=".lc($dbk);
-	    $out .= " --btw=".lc(BTWTYPES->[$btw]) unless $btw == BTWTYPE_NORMAAL;
+	    $out .= _xt("cmd:relatie")." --".__xt("cmo:relatie:dagboek")."=".lc($dbk);
+	    $out .= " --".__xt("cmo:relatie:btw")."=".lc(BTWTYPES->[$btw]) unless $btw == BTWTYPE_NORMAAL;
 	}
 	$out .= " \\\n        ";
 	$out .= sprintf("%-12s %-40s %d", _quote($code), _quote($desc), $acct);
@@ -171,13 +168,13 @@ sub _opening {
 
     my $sth;
 
-    my $out = __x("# {what} voor administratie {adm}",
-		  what => _T("Openingsgegevens"), adm => $dbh->adm("name")) . "\n" .
-	      __x("# Aangemaakt door {id} op {date}",
-		  id => $EB::ident, date => datefmt_full(iso8601date())) . "\n" .
+    my $out = "# " . __x("{what} voor administratie {adm}",
+			 what => _T("Openingsgegevens"), adm => $dbh->adm("name")) . "\n" .
+	      "# " . __x("Aangemaakt door {id} op {date}",
+			 id => $EB::ident, date => datefmt_full(iso8601date())) . "\n" .
 	      "# Content-Type: text/plain; charset = UTF-8\n\n";
 
-    $out .= "adm_naam         " . _quote($dbh->adm("name")) . "\n";
+    $out .= _xt("cmd:adm_naam") . "         " . _quote($dbh->adm("name")) . "\n";
 
     my $begin = $dbh->do("SELECT min(bky_begin)".
 			 " FROM Boekjaren".
@@ -185,10 +182,10 @@ sub _opening {
 			 BKY_PREVIOUS);
     $begin = $begin->[0];
 
-    $out .= "adm_begindatum   " . substr($begin, 0, 4) . "\n";
-    $out .= "adm_boekjaarcode " . _quote($dbh->lookup($begin, qw(Boekjaren bky_begin bky_code))) . "\n";
-    $out .= "adm_btwperiode   " .
-      (qw(geen jaar x x kwartaal x x x x x x x maand)[$dbh->lookup($begin, qw(Boekjaren bky_begin bky_btwperiod))]).
+    $out .= _xt("cmd:adm_begindatum") . "   " . substr($begin, 0, 4) . "\n";
+    $out .= _xt("cmd:adm_boekjaarcode") . " " . _quote($dbh->lookup($begin, qw(Boekjaren bky_begin bky_code))) . "\n";
+    $out .= _xt("cmd:adm_btwperiode") . "   " .
+      _T(qw(geen jaar x x kwartaal x x x x x x x maand)[$dbh->lookup($begin, qw(Boekjaren bky_begin bky_btwperiod))]).
 	"\n" if $dbh->does_btw;
 
     $out .= "\n# " . _T("Openingsbalans") . "\n";
@@ -216,7 +213,8 @@ sub _opening {
 	if ( !defined($debcrd) || $acc_debcrd != $debcrd ) {
 	    $out .= "\n# " . ($acc_debcrd ? _T("Debet") : _T("Credit")) . "\n";
 	}
-	$out .= sprintf("adm_balans %-5s %10s   # %s\n",
+	$out .= sprintf("%s %-5s %10s   # %s\n",
+			_xt("cmd:adm_balans"),
 			$acc_id, numfmt_plain($acc_balance),
 			$acc_desc);
 	$debcrd = $acc_debcrd;
@@ -226,7 +224,7 @@ sub _opening {
 		arg1 => numfmt_plain($dt),
 		arg2 => numfmt_plain($ct))."\n")
       unless $dt == $ct;
-    $out .= "\n# " .  _T("Totaal") . "\n" . "adm_balanstotaal " . numfmt_plain($dt) . "\n";
+    $out .= "\n# " .  _T("Totaal") . "\n" . _xt("cmd:adm_balanstotaal") . " " . numfmt_plain($dt) . "\n";
 
     $sth = $dbh->sql_exec("SELECT bsk_id".
 			  " FROM Boekstukken".
@@ -255,7 +253,7 @@ sub _opening {
 	$rr = $sth->fetchrow_arrayref;
     }
 
-    $out .= "\n# "._T("Openen van de administratie")."\n\nadm_open\n";
+    $out .= "\n# "._T("Openen van de administratie")."\n\n"._xt("cmd:adm_open")."\n";
     $out .= "\n# " . __x("Einde {what}", what => _T("Openingsgegevens")) . "\n";
     $out;
 }
@@ -263,10 +261,10 @@ sub _opening {
 sub _mutaties {
     my ($self, $opts) = @_;
 
-    my $out = __x("# {what} voor administratie {adm}",
-		  what => _T("Boekingen"), adm => $dbh->adm("name")) . "\n" .
-	      __x("# Aangemaakt door {id} op {date}",
-		  id => $EB::ident, date => datefmt_full(iso8601date())) . "\n" .
+    my $out = "# " . __x("{what} voor administratie {adm}",
+			 what => _T("Boekingen"), adm => $dbh->adm("name")) . "\n" .
+	      "# " . __x("Aangemaakt door {id} op {date}",
+			 id => $EB::ident, date => datefmt_full(iso8601date())) . "\n" .
 	      "# Content-Type: text/plain; charset = UTF-8\n\n";
 
     my @bky;
@@ -282,7 +280,9 @@ sub _mutaties {
     my $check_je = sub {
 	my ($bky) = @_;
 	if ( $dbh->lookup($bky, qw(Boekjaren bky_code bky_closed)) ) {
-	    $out .= "jaareinde --boekjaar=" . _quote($bky) . " --definitief\n";
+	    $out .= _xt("cmd:jaareinde") .
+	      " --" . __xt("cmo:jaareinde:boekjaar") . "=" . _quote($bky) .
+	      " --" . __xt("cmo:jaareinde:definitief") . "\n";
 	}
 	else {
 	    $sth = $dbh->sql_exec("SELECT COUNT(*)".
@@ -290,7 +290,8 @@ sub _mutaties {
 				  " WHERE bkb_bky = ?", $bky);
 	    my $rr;
 	    if ( ($rr = $sth->fetchrow_arrayref) && $rr->[0] ) {
-		$out .= "jaareinde --boekjaar=" . _quote($bky) . "\n";
+		$out .= _xt("cmd:jaareinde") .
+		  " --" . __xt("cmo:jaareinde:boekjaar") . "=" . _quote($bky) . "\n";
 	    }
 	    $sth->finish;
 	}
@@ -300,8 +301,11 @@ sub _mutaties {
 	    my $bkb = $dbh->lookup($bky, qw(Boekjaren bky_code bky_begin));
 	    if ( $bb gt $bkb ) {
 		$bke = parse_date($bb, undef, -1) if $bb le $bke;
-		$out .= "btwaangifte --periode=".
-		  datefmt_full($bkb)."-".datefmt_full($bke)." --definitief --noreport\n";
+		$out .= _xt("cmd:btwaangifte").
+		  " --" . __xt("cmo:btwaangifte:periode") . "=".
+		  datefmt_full($bkb)."-".datefmt_full($bke) .
+		  " --" . __xt("cmo:btwaangifte:definitief") .
+		  " --" . __xt("cmo:btwaangifte:noreport") . "\n";
 	    }
 	}
     };
@@ -313,12 +317,12 @@ sub _mutaties {
 	    $check_je->($cur_bky);
 	    my $bp = $dbh->lookup($bky, qw(Boekjaren bky_code bky_btwperiod));
 	    $out .= "\n# ". _T("Openen nieuw boekjaar") . "\n\n";
-	    $out .= "adm_boekjaarcode " . _quote($bky) . "\n";
-	    $out .= "adm_btwperiode " . lc(BTWPERIODES->[$bp]) . "\n" if $bp;
-	    $out .= "adm_open\n";
+	    $out .= _xt("cmd:adm_boekjaarcode") . " " . _quote($bky) . "\n";
+	    $out .= _xt("cmd:adm_btwperiode") . " " . lc(BTWPERIODES->[$bp]) . "\n" if $bp;
+	    $out .= _xt("cmd:adm_open") . "\n";
 	    $cur_bky = $bky;
 	}
-	$out .= "boekjaar " . _quote($bky) . "\n";
+	$out .= _xt("cmd:boekjaar") . " " . _quote($bky) . "\n";
 
 	$sth = $dbh->sql_exec("SELECT bsk_id, dbk_id".
 			      " FROM Boekstukken, Dagboeken".
@@ -350,5 +354,10 @@ sub _mutaties {
     $out;
 }
 
+sub _xt {			# scm:btw -> scm:vat -> vat
+    my $t = _T(shift);
+    $t =~ s/^.*://;
+    $t;
+}
 
 1;
