@@ -2,12 +2,11 @@
 
 use utf8;
 
-# RCS Id          : $Id: Opening.pm,v 1.42 2009/10/14 21:14:02 jv Exp $
 # Author          : Johan Vromans
 # Created On      : Tue Aug 30 09:49:11 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Wed Oct 14 23:10:37 2009
-# Update Count    : 296
+# Last Modified On: Fri Aug 31 18:18:52 2012
+# Update Count    : 308
 # Status          : Unknown, Use with caution!
 
 package main;
@@ -20,8 +19,6 @@ package EB::Tools::Opening;
 use strict;
 use warnings;
 
-our $VERSION = sprintf "%d.%03d", q$Revision: 1.42 $ =~ /(\d+)/g;
-
 use EB;
 use EB::Format;
 
@@ -316,13 +313,13 @@ sub open {
 			 rcrd => numfmt(-$rcredit)).
 		     ($rdebet == -$rcredit
 		      ? __x(" (balanstotaal {total})", total => numfmt($o->{balanstotaal} - $rdebet))
-		      : ())."\n");
+		      : '')."\n");
 		warn(__x("Boekhoudkundig residu debet = {rdeb}, credit = {rcrd}",
 			 rdeb => numfmt($bdebet),
 			 rcrd => numfmt(-$bcredit)).
 		     ($bdebet == -$bcredit
 		      ? __x(" (balanstotaal {total})", total => numfmt($o->{balanstotaal} - $bdebet))
-		      : ())."\n");
+		      : '')."\n");
 	    }
 
 	    # Helpful hints...
@@ -526,7 +523,9 @@ sub reopen {
 
     # New begin date is old + one year.
     my $y = parse_date($dbh->adm("end"), undef, 1);
-    if ( $y gt iso8601date() ) {
+
+    # Avoid opening a new book year too soon.
+    if ( $y gt parse_date(iso8601date(), undef, 0, 6) ) {
 	warn(__x("Begindatum {year} komt in de toekomst te liggen",
 		 year => $y)."\n");
 	$fail++;
@@ -579,7 +578,7 @@ sub reopen {
 
 sub shellhelp {
     my ($self, $cmd) = @_;
-    <<EOS;
+    _T( <<EOS );
 Het openen van een administratie kan slechts éénmaal gebeuren, vóór
 het invoeren van de eerste mutatie. Het openen van een nieuw boekjaar
 kan te allen tijde worden uitgevoerd, uiteraard maar één keer per
@@ -594,7 +593,7 @@ Mogelijke opdrachten voor openen van een boekjaar:
 
   adm_btwperiode [ jaar | kwartaal | maand ]
   adm_boekjaarcode <code>
-                Een code van max 4 letters en/of cijfers waarmee het
+		Een code van max 4 letters en/of cijfers waarmee het
 		boekjaar kan worden geïdentificeerd.
 		Standaard wordt het jaartal van het boekjaar genomen.
   adm_open
@@ -609,7 +608,7 @@ Opdrachten voor het openen van een administratie:
 		Een administratie loopt altijd van 1 januari tot en
 		met 31 december van een kalenderjaar.
   adm_boekjaarcode <code>
-                Een code van max 4 letters en/of cijfers waarmee het
+		Een code van max 4 letters en/of cijfers waarmee het
 		boekjaar kan worden geïdentificeerd.
 		Standaard wordt het jaartal van het boekjaar genomen.
 		De boekjaarcode is alleen relevant indien er meerdere
@@ -617,7 +616,7 @@ Opdrachten voor het openen van een administratie:
   adm_balanstotaal <bedrag>
 		Als een balanstotaal is opgegeven, moeten er ook
 		openingsbalansboekingen worden uitgevoerd met een of
-                meer adm_balans opdrachten.
+		meer adm_balans opdrachten.
   adm_balans <balansrekening> <bedrag>
 		De debet en credit boekingen moeten uiteindelijk
 		allebei gelijk zijn aan het opgegeven balanstotaal.
@@ -628,7 +627,7 @@ Opdrachten voor het openen van een administratie:
   adm_relatie <boekstuk> <datum> <code> <omschrijving> <bedrag>
 		Invoeren van een openstaande post uit het voorgaande
 		boekjaar. Het <boekstuk> moet volledig zijn, dus
-                <dagboek>:<boekjaar>:<nummer>.
+		<dagboek>:<boekjaar>:<nummer>.
   adm_open
 		Alle informatie die met de bovenstaande opdrachten is
 		ingevoerd, wordt verwerkt.
@@ -636,7 +635,7 @@ EOS
 }
 
 sub help_btwperiode {
-    <<EOS;
+    _T( <<EOS );
 Specifeer de BTW aangifteperiode voor het nieuw te openen jaar.
 
   adm_btwperiode [ jaar | kwartaal | maand ]
@@ -647,7 +646,7 @@ EOS
 }
 
 sub help_boekjaarcode {
-    <<EOS;
+    _T( <<EOS );
 Specificeert de boekjaarcode voor het nieuw te openen jaar.
 
   adm_boekjaarcode <code>
@@ -663,8 +662,8 @@ EOS
 }
 
 sub help_naam {
-    <<EOS;
-Specificeert de naam van de administatie, te gebruiken voor rapportages.
+    _T( <<EOS );
+Specificeert de naam van de administratie, te gebruiken voor rapportages.
 
   adm_naam "Naam van de administratie"
 
@@ -674,7 +673,7 @@ EOS
 }
 
 sub help_begindatum {
-    <<EOS;
+    _T( <<EOS );
 Specificeert de begindatum van de administratie.
 Een administratie loopt altijd van 1 januari tot en met 31 december
 van een kalenderjaar. Daarom moet als begindatum een jaartal worden
@@ -688,7 +687,7 @@ EOS
 }
 
 sub help_balanstotaal {
-    <<EOS;
+    _T( <<EOS );
 Specificeert het balanstotaal voor de in te voeren openingbalans.
 
   adm_balanstotaal <bedrag>
@@ -704,7 +703,7 @@ EOS
 }
 
 sub help_balans {
-    <<EOS;
+    _T( <<EOS );
 Specificeert een balanspost voor de openingsbalans.
 
   adm_balans <balansrekening> <bedrag>
@@ -723,7 +722,7 @@ EOS
 }
 
 sub help_relatie {
-    <<EOS;
+    _T( <<EOS );
 Specificeert een openstaande post uit een voorgaand boekjaar.
 
   adm_relatie <boekstuk> <datum> <code> <omschrijving> <bedrag>
